;	This program is free software; you can redistribute it and/or modify
;	it under the terms of the GNU General Public License as published by
;	the Free Software Foundation; either version 2 of the License, or
;	(at your option) any later version.
;
;	This program is distributed in the hope that it will be useful,
;	but WITHOUT ANY WARRANTY; without even the implied warranty of
;	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;	GNU General Public License for more details.
;
;	You should have received a copy of the GNU General Public License
;	along with this program; if not, write to the Free Software
;	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
; Interface routines...main RPN loop
;
Interface_Stack_Numbers:
	DB "5:",0,"4:",0,"3:",0,"2:",0,"1:",0
Interface_ShowStack_InScientific:
	ld a,3
	ld (curcol),a
	call Scientific_Notation
	jp Interface_ShowStack_AfterDisplay
Interface_Next:
Interface_Clear:
	B_CALL ClrScrnFull
Interface:
	B_CALL RunIndicOff
	res indicOnly,(iy+indicFlags)
	ld hl,(mathAreaStart)
	ld (areaFreeStart),hl
	ld (errorStack),sp
	ld hl,ErrorHandler
	ld (errorPointer),hl
	;Set up entry
	ld hl,(areaFreeEnd)
	ld de,(areaFreeStart)
	or a
	sbc hl,de
	;jp z,App_Exit				;perhaps fix this? No memory at all!
	ld (inputMax),hl		;maximum allowable input
	ld (inputStart),de
	ld (inputCur),de
	ld bc,0
	ld (inputLen),bc
	;Display the stack
	ld hl,0
	ld (currow),hl
	ld hl,Interface_Stack_Numbers
	ld b,5
Interface_ShowStack_Numbers:
	call putstr
	xor a
	ld (curcol),a
	ld a,(currow)
	inc a
	ld (currow),a
	djnz Interface_ShowStack_Numbers
	ld a,(rpnEntryCount)
	cp 5
	jr c,Interface_ShowStack_Stack_Count
	ld a,5
Interface_ShowStack_Stack_Count:
	or a
	jr z,Interface_ShowStack_Done
	;A is the number of entries to display
	ld b,a
	ld a,4
	sub b
	ld (currow),a
	ld hl,(rpnNextEntry)
	ld e,b
	sla e
	ld d,0
	or a
	sbc hl,de
	ex de,hl
Interface_ShowStack_Loop:
	ld hl,currow
	inc (hl)
	ex de,hl
	ld e,(hl)
	inc hl
	ld d,(hl)
	ex de,hl
	push bc
	push de
	bit flagScientific,(iy+asm_Flag1)
	jp nz,Interface_ShowStack_InScientific
	call BCD_Display
Interface_ShowStack_AfterDisplay:
	pop de
	pop bc
	inc de
	djnz Interface_ShowStack_Loop
Interface_ShowStack_Done:
	ld a,(rpnEntryCount)
	cp 6
	jr c,Interface_ShowStack_NoArrow
	ld hl,1*256
	ld (currow),hl
	ld a,LupArrow
	B_CALL PutC
Interface_ShowStack_NoArrow:
	ld hl,6
	ld (currow),hl
	;Ready for input
Interface_Input:
	in a,(4)
	bit 3,a
	jr nz,Interface_Input_KeyProcess_NotON
	;Fast scroll
	ld a,0FFh
	out (1),a
	nop
	nop
	ld a,0FEh
	out (1),a
	nop
	nop
	in a,(1)
	cp 0FBh
	jp z,Interface_ScrollRight
	cp 0FDh
	jp z,Interface_ScrollLeft
Interface_Input_KeyProcess_NotON:
	bit flagError,(iy+asm_Flag1)		;if set there is a key code already in (errorkeycode)
	ld a,(errorkeycode)
	jr nz,Interface_Input_KeyProcess
	B_CALL GetKey
Interface_Input_KeyProcess:
	res flagError,(iy+asm_Flag1)
	res onInterrupt,(iy+onFlags)
	;The only valid keys are:
	;2nd+Quit, Clear, Del, 0-9, negative sign
	;Addition, subtraction, division, multiplication, exponent signs
	;Math key, but we'll handle that later
	cp 40h					;2nd+quit
	jp z,App_Exit
	cp kClear				;clear
	jp z,Interface_Clear_Key
	cp kDel
	jp z,Interface_Backspace
	cp kLeft
	jp z,Interface_ScrollLeft
	cp kRight
	jp z,Interface_ScrollRight
	cp kUp
	jp z,Interface_ScrollUp
	cp kDown
	jp z,Interface_ScrollDown
	cp kVarx
	jp z,Interface_SwapValues
	;cp kAppsMenu
	;jp z,Interface_Games
	cp kMath
	jp z,FunctionsMenu
	cp kWindow
	jp z,ToggleSciNotation
	cp kStore
	jp z,Export
	cp kEE
	jp z,Show_EE
	cp kEnter
  	jp z,Interface_Enter
	cp 80h
	jr c,Interface_Input_NotOperation
	cp 8Ch
	jp z,Parse_Operation
	cp 85h
	jp c,Parse_Operation
	cp kSquare		;square
	jp z,Parse_Operation
	cp kSqrt		;square root
	jp z,Parse_Operation
	cp kALog		;10^x
	jp z,Parse_Operation
	cp kLog		;logarithm
	jp z,Parse_Operation
Interface_Input_NotOperation:
	cp 8Eh
	jp c,Interface_Input
	cp 98h
	jp nc,Interface_Input
	;Numeric key!
	bit flagInputActive,(iy+asm_Flag1)
	jr nz,Interface_ByteAdd
	;Set up numeric input
	set flagInputActive,(iy+asm_Flag1)
	B_CALL CursorOn
Interface_ByteAdd:
	ld hl,(inputMax)
	ld bc,(inputLen)
	or a
	sbc hl,bc
	jp z,Interface_Input				;out of memory
	inc bc
	ld (inputLen),bc
	ld hl,(inputCur)
	ld (hl),a
	inc hl
	ld (inputCur),hl
	ld e,a
	ld d,0
	ld a,(curcol)
	cp 15
	jr c,Interface_ByteAdd_Okay
	;On the last edge...scroll it
	ld hl,(inputCur)
	dec hl
	dec hl
	ld b,14
	ld a,b
	dec a
	ld (curcol),a
Interface_ByteAdd_LoopBack:
	push bc
	ld d,0
	ld e,(hl)
	push hl
	B_CALL KeyToString
	B_CALL PutPSB
	ld hl,curcol
	dec (hl)
	dec (hl)
	pop hl
	dec hl
	pop bc
	djnz Interface_ByteAdd_LoopBack
	ld a,14
	ld (curcol),a
	ld hl,(inputCur)
	dec hl
	ld e,(hl)
Interface_ByteAdd_Okay:
	B_CALL KeyToString
	B_CALL PutPSB
	jp Interface_Input	
Interface_Clear_Key:
	ld bc,(inputLen)
	ld a,b
	or c
	jr z,Interface_Backspace_DropStackValue
	res flagInputActive,(iy+asm_Flag1)
	B_CALL CursorOff
	jp Interface_Clear
Interface_Backspace_DropStackValue:
	res flagInputActive,(iy+asm_Flag1)
	B_CALL CursorOff
	ld a,(rpnEntryCount)
	or a
	jp z,Interface_Next
	dec a
	ld (rpnEntryCount),a
	ld hl,(rpnNextEntry)
	dec hl
	ld d,(hl)
	dec hl
	ld e,(hl)
	ex de,hl
	ld c,(hl)
	inc hl
	ld b,(hl)
	res 7,b
	add hl,bc
	ld (areaFreeEnd),hl
	ld (rpnNextEntry),de
	jp Interface_Next
Interface_ScrollDown:
	;Rotate the stack downwards
	bit flagInputActive,(iy+asm_Flag1)
	jp nz,Interface_Input
	ld a,(rpnEntryCount)
	cp 2
	jp c,Interface_Input
	ld hl,(rpnNextEntry)
	dec hl
	ld d,(hl)
	dec hl
	ld e,(hl)
	ex de,hl
	push hl
	ld c,(hl)
	inc hl
	ld b,(hl)
	res 7,b
	inc bc
	inc bc
	pop hl
	or a
	sbc hl,bc
	ex de,hl
	ld hl,(areaFreeStart)
	or a
	sbc hl,de
	jp nc,Interface_Input
	;Now copy the entire stack down in memory a bit so we can put the top stack value
	;at the bottom
	ld (temp3),bc			;temp3 is the offset to shift
	ld hl,rpnStackStart
	ld e,(hl)
	inc hl
	ld d,(hl)
	ex de,hl
	ld c,(hl)
	inc hl
	ld b,(hl)
	res 7,b
	add hl,bc
	;HL is the last byte of the shift
	ld (temp2),hl
	ld hl,(rpnNextEntry)
	dec hl
	ld d,(hl)
	dec hl
	ld e,(hl)
	or a
	ld hl,(temp2)
	sbc hl,de
	inc hl
	push hl
	ld h,d
	ld l,e
	or a
	ld bc,(temp3)
	sbc hl,bc
	pop bc
	ex de,hl
	push de
	ldir
	;Now copy the top entry to the bottom
	ld hl,(temp2)
	ld bc,(temp3)
	or a
	sbc hl,bc
	ex de,hl
	pop hl
	inc de
	push de
	ldir
	;The stack is now shifted...did we update the stack pointers? Not yet.
	ld hl,(rpnNextEntry)
	dec hl
	ld d,h
	ld e,l
	dec hl
	dec hl
	ld a,(rpnEntryCount)
	dec a
	add a,a
	ld b,0
	ld c,a
	lddr
	;Spot on. Let's adjust the pointers now.
	ld hl,rpnStackStart
	pop de
	ld (hl),e
	inc hl
	ld (hl),d
	inc hl
ScrollUp_Loop:
	ld e,(hl)
	inc hl
	ld d,(hl)
	ex de,hl
	ld bc,(temp3)
	or a
	sbc hl,bc
	ex de,hl
	ld (hl),d
	dec hl
	ld (hl),e
	inc hl
	inc hl
	ld de,(rpnNextEntry)
	or a
	sbc hl,de
	jp z,Interface_Next
	add hl,de
	jr ScrollUp_Loop
Interface_ScrollUp:
	;Rotate the stack upwards
	bit flagInputActive,(iy+asm_Flag1)
	jp nz,Interface_Input
	ld a,(rpnEntryCount)
	cp 2
	jp c,Interface_Input
	ld hl,rpnStackStart
	ld e,(hl)
	inc hl
	ld d,(hl)
	ex de,hl
	ld c,(hl)
	inc hl
	ld b,(hl)
	res 7,b
	ld hl,(rpnNextEntry)
	dec hl
	ld d,(hl)
	dec hl
	ld e,(hl)
	ex de,hl
	scf
	sbc hl,bc
	ex de,hl
	ld hl,(areaFreeStart)
	or a
	sbc hl,de
	jp nc,Interface_Input
	;Now copy down the bottom stack value past the top stack value
	ld hl,rpnStackStart
	ld e,(hl)
	inc hl
	ld d,(hl)
	ex de,hl
	;HL is where to copy from
	push hl
	dec hl
	ld (temp1),hl				;temp1 is the last byte that needs to be copied
	inc hl
	ld c,(hl)
	inc hl
	ld b,(hl)
	res 7,b
	inc bc
	inc bc	;BC is the length of the entry
	;Now figure out where to copy the entry to
	ld hl,(rpnNextEntry)
	dec hl
	ld d,(hl)
	dec hl
	ld e,(hl)
	ex de,hl
	;HL is the bottom entry...
	ld (temp4),hl			;temp storage
	or a
	sbc hl,bc
	;HL is now where to copy it to
	ld (temp2),hl			;temp2 is the first byte that needs to be copied
	ld (temp3),bc			;temp3 is the offset of the shift
	ex de,hl
	pop hl
	ldir
	ld hl,rpnStackStart
	ld d,h
	ld e,l
	inc hl
	inc hl
	ld a,(rpnEntryCount)
	dec a
	add a,a
	ld b,0
	ld c,a
	ldir
	ld hl,(rpnNextEntry)
	ld de,(temp4)
	dec hl
	ld (hl),d
	dec hl
	ld (hl),e
	dec hl
ScrollStackUp_Loop:
	ld d,(hl)
	dec hl
	ld e,(hl)
	ex de,hl
	ld bc,(temp3)
	add hl,bc
	ex de,hl
	ld (hl),e
	inc hl
	ld (hl),d
	dec hl
	ld de,rpnStackStart
	or a
	sbc hl,de
	jr z,ScrollStackUp_Done
	add hl,de
	dec hl
	jr ScrollStackUp_Loop
ScrollStackUp_Done:
	;Now shift the stack up
	ld hl,(temp1)
	ld de,(temp2)
	or a
	sbc hl,de
	inc hl
	;HL is the length of the shift
	ld b,h
	ld c,l
	ld de,(temp3)
	ld hl,(temp1)
	add hl,de
	ld de,(temp1)
	ex de,hl
	lddr
	jp Interface_Next
Interface_SwapValues:
	bit flagInputActive,(iy+asm_Flag1)
	jp nz,Interface_Input
	ld a,(rpnEntryCount)
	cp 2
	jp c,Interface_Input
	ld hl,(rpnNextEntry)
	dec hl
	ld d,(hl)
	dec hl
	ld e,(hl)
	dec hl
	ld a,(hl)
	dec hl
	ld l,(hl)
	ld h,a
	;DE is the pointer to top value
	;HL points to second value
	push de
	push hl
	push hl
	;Let's copy down the second value past the top.
	;After this we will simply copy these two values back up so that the top value
	;slides over where the second one was, and the second one where the first was.
	ld c,(hl)
	inc hl
	ld b,(hl)
	res 7,b
	dec hl
	inc bc
	inc bc
	ex de,hl
	or a
	sbc hl,bc
	ex de,hl
	ld hl,(areaFreeStart)
	or a
	sbc hl,de
	jp nc,Interface_Input
	push hl
	push de
	ld hl,(rpnNextEntry)
	dec hl
	ld d,(hl)
	dec hl
	ld e,(hl)
	ex de,hl
	add hl,bc
	ex de,hl
	dec hl
	dec hl
	ld (hl),e
	inc hl
	ld (hl),d
	pop de
	pop hl
	;DE is the byte to copy the second value to
	;BC is the length of the copy
	pop hl
	ldir		;copy
	;Now we must shift this up
	pop hl
	ld ix,0
	ld c,(hl)
	inc hl
	ld b,(hl)
	res 7,b
	add hl,bc
	add ix,bc
	inc ix
	inc ix
	;HL is now the last byte of the original second value
	ex de,hl
	pop hl
	ld c,(hl)
	inc hl
	ld b,(hl)
	res 7,b
	add hl,bc
	add ix,bc
	inc ix
	inc ix
	;HL is now the last byte of the original top value
	push ix
	pop bc
	lddr		;copy...
	inc de
	ld hl,(rpnNextEntry)
	dec hl
	ld (hl),d
	dec hl
	ld (hl),e
	ex de,hl
	ld c,(hl)
	inc hl
	ld b,(hl)
	scf
	adc hl,bc
	ex de,hl
	dec hl
	ld (hl),d
	dec hl
	ld (hl),e
	jp Interface_Next
Interface_ScrollLeft:
	bit flagInputActive,(iy+asm_Flag1)
	jr nz,Interface_Backspace
	ld a,(rpnEntryCount)
	or a
	jp z,Interface_Input
	;Scroll the number
	ld hl,(currow)
	push hl
	ld a,4
	ld (currow),a
	call BCD_Display_Left
	pop hl
	ld (currow),hl
	jp Interface_Input
Interface_ScrollRight:
	bit flagInputActive,(iy+asm_Flag1)
	jp nz,Interface_Input
	ld a,(rpnEntryCount)
	or a
	jp z,Interface_Input
	ld hl,(currow)
	push hl
	ld a,4
	ld (currow),a
	call BCD_Display_Right
	pop hl
	ld (currow),hl
	jp Interface_Input
Interface_Backspace:
	;Backspace!
	cp kLeft
	jr z,Interface_Backspace_NoDropEver
	ld hl,(inputLen)
	ld a,h
	or l
	jp z,Interface_Backspace_DropStackValue
Interface_Backspace_NoDropEver:
	ld hl,(inputLen)
	ld a,h
	or l
	jp z,Interface_Input
	ld de,16
	or a
	sbc hl,de
	jr c,Interface_Backspace_Normal
	;shift the displayed number back
	B_CALL CursorOff
	ld hl,(inputCur)
	ld bc,16
	or a
	sbc hl,bc
	ld b,15
	xor a
	ld (curcol),a
Interface_Backspace_ShiftDisplay:
	push bc
	ld d,0
	ld e,(hl)
	push hl
	B_CALL KeyToString
	B_CALL PutPSB
	pop hl
	inc hl
	pop bc
	djnz Interface_Backspace_ShiftDisplay
Interface_Backspace_ShiftDone:
	ld (inputCur),hl
	ld bc,(inputLen)
	dec bc
	ld (inputLen),bc
	B_CALL EraseEOL
	ld bc,(inputLen)
	ld a,b
	or c
	jr nz,Interface_Backspace_Done
	res flagInputActive,(iy+asm_Flag1)
	jp Interface_Input
Interface_Backspace_Normal:
	ld bc,(inputLen)
	dec bc
	ld (inputLen),bc
	ld hl,(inputCur)
	dec hl
	ld (inputCur),hl
	B_CALL CursorOff
	ld hl,curcol
	dec (hl)
	ld a,' '
	B_CALL PutMap
	ld bc,(inputLen)
	ld a,b
	or c
	jr nz,Interface_Backspace_Done
	res flagInputActive,(iy+asm_Flag1)
	jp Interface_Input
Interface_Backspace_Done:
	B_CALL CursorOn
	jp Interface_Input
ToggleSciNotation:
	bit flagInputActive,(iy+asm_Flag1)
	jp nz,Interface_Input
	ld a,(iy+asm_Flag1)
	xor (1<<(flagScientific))
	ld (iy+asm_Flag1),a
	jp Interface_Next
Show_EE:
	bit flagInputActive,(iy+asm_Flag1)
	jp nz,Interface_Input
	ld a,(rpnEntryCount)
	cp 1
	jp c,Interface_Input
	ld hl,2*256+4
	ld (currow),hl
	B_CALL EraseEOL
	ld a,3
	ld (curcol),a
	ld hl,(rpnNextEntry)
	dec hl
	ld d,(hl)
	dec hl
	ld e,(hl)
	ex de,hl
	call Scientific_Notation
	B_CALL GetKey
	push af
	ld a,4
	ld (currow),a
	ld hl,(rpnNextEntry)
	dec hl
	ld d,(hl)
	dec hl
	ld e,(hl)
	ex de,hl
	bit flagScientific,(iy+asm_Flag1)
	call z,BCD_Display
	ld hl,6
	ld (currow),hl
	pop af
	jp Interface_Input_KeyProcess
;Interface_Games:
;	bit flagInputActive,(iy+asm_Flag1)
;	call nz,Parse_Ret
;	B_CALL CursorOff
;	res flagInputActive,(iy+asm_Flag1)
;	jp Games
Interface_Enter:
	B_CALL CursorOff
	res flagInputActive,(iy+asm_Flag1)
	ld bc,(inputLen)
	ld a,b
	or c
	jr nz,Parse
	ld a,(rpnEntryCount)
	or a
	jp z,Interface_Input
	cp maxStackEntries
	jp nc,StackCountError
	;Duplicate the last stack value
	ld hl,(rpnNextEntry)
	dec hl
	ld d,(hl)
	dec hl
	ld e,(hl)
	ex de,hl
	call BCD_Clone
	;HL is the number...
	ld c,(hl)
	inc hl
	ld b,(hl)
	ld a,b
	res 7,b
	dec hl
	push hl
	ld hl,(areaFreeEnd)
	or a
	sbc hl,bc
	ld de,(areaFreeStart)
	inc de
	inc de
	inc de
	or a
	sbc hl,de
	jp c,MemoryError
	ld hl,(areaFreeEnd)
	dec hl
	scf
	sbc hl,bc
	ld (areaFreeEnd),hl
	inc hl
	ld (hl),c
	inc hl
	sla b
	ld d,a
	sla d
	rr b
	ld (hl),b
	inc hl
	pop de
	inc de
	inc de
	push hl
	ex de,hl
	ld b,a
	res 7,b
	ldir
	ld hl,rpnEntryCount
	inc (hl)
	ld hl,(rpnNextEntry)
	pop de
	dec de
	dec de
	ld (hl),e
	inc hl
	ld (hl),d
	inc hl
	ld (rpnNextEntry),hl
	jp Interface_Next
Parse:
	ld hl,Interface_Next
	push hl
Parse_Ret:
	ld a,(rpnEntryCount)
	cp maxStackEntries
	jp nc,StackCountError
	ld bc,(inputLen)
	ld a,b
	or c
	ret z
	ld bc,(inputLen)
	ld hl,(inputStart)
	ld a,(hl)
	cp 8Ch
	jp c,ParseError
	cp 98h
	jp nc,ParseError
	cp 8Eh	;zero
	jr nz,Parse_Ret_NoZeroes
Parse_Ret_RemoveZeroes:
	push hl
	ld hl,1
	or a
	sbc hl,bc
	pop hl
	jr z,Parse_Ret_NoZeroes
	ld a,(hl)
	cp 8Eh
	jr nz,Parse_Ret_NoZeroes
	inc hl
	dec bc
	jr Parse_Ret_RemoveZeroes
Parse_Ret_NoZeroes:
	;Let's parse a number!
	;BC is the length of the bytes...
	ld (inputLen),bc
	ld (inputStart),hl
	push hl
	ld hl,(areaFreeEnd)
	or a
	sbc hl,bc
	ld de,(areaFreeStart)
	inc de
	inc de
	inc de
	or a
	sbc hl,de
	jp c,MemoryError
	xor a
	ld (temp1),a		;sign of the number...
	ld hl,(inputStart)
	ld a,(hl)
	cp 8Ch
	jr nz,Parse_Number_Positive
	ld a,80h
	ld (temp1),a
	ld bc,(inputLen)
	dec bc
	ld (inputLen),bc
	pop hl
	inc hl
	push hl
Parse_Number_Positive:
	srl b
	rr c
	jr nc,Parse_Number_Even
	inc bc
	ld hl,(areaFreeEnd)
	dec hl
	or a
	sbc hl,bc
	dec hl
	ld (areaFreeEnd),hl
	inc hl
	ld (hl),c
	inc hl
	ld (hl),b
	inc hl
	pop de
Parse_Number_Odd_Loop:
	ld a,(de)
	sub 8Eh
	jp c,ParseError
	cp 10
	jp nc,ParseError
	ld (hl),a
	inc hl
	inc de
	dec bc
	ld a,b
	or c
	jr z,Parse_Number_Done
	ld bc,(inputLen)
	dec bc
	ld (inputLen),bc
	jr Parse_Number_Ready
Parse_Number_Even:
	ld hl,(areaFreeEnd)
	dec hl
	or a
	sbc hl,bc
	dec hl
	ld (areaFreeEnd),hl
	inc hl
	ld (hl),c
	inc hl
	ld (hl),b
	inc hl
	pop de
Parse_Number_Ready:
	ld bc,(inputLen)
	bit 0,c
	jr z,Parse_Number_Loop
	inc bc
Parse_Number_Loop:
	ld a,(de)
	sub 8Eh
	jp c,ParseError
	cp 10
	jp nc,ParseError
	bit 0,c
	jr z,Parse_Number_Loop_High
	or (hl)
	ld (hl),a
	inc de
	inc hl
	dec bc
	ld a,b
	or c
	jr nz,Parse_Number_Loop
	jr Parse_Number_Done
Parse_Number_Loop_High:
	sla a
	sla a
	sla a
	sla a
	ld (hl),a
	inc de
	dec bc
	ld a,b
	or c
	jr nz,Parse_Number_Loop
Parse_Number_Done:
	;all done...
	ld de,(areaFreeEnd)
	inc de
	ld hl,(rpnNextEntry)
	ld (hl),e
	inc hl
	ld (hl),d
	inc hl
	ld (rpnNextEntry),hl
	ld hl,rpnEntryCount
	inc (hl)
	;DE is the first byte
	inc de
	ld a,(temp1)
	ld b,a
	ld a,(de)
	or b
	ld (de),a
	ret
Interface_Functions:
	bit flagInputActive,(iy+asm_Flag1)
	jr z,Interface_Functions_Jump
	B_CALL CursorOff
	call Parse_Ret
Interface_Functions_Jump:
	jp FunctionsMenu
Parse_Operation:
	bit flagInputActive,(iy+asm_Flag1)
	jr z,Parse_Operation_Normal
	res flagInputActive,(iy+asm_Flag1)
	push af
	B_CALL CursorOff
	call Parse_Ret
	pop af
Parse_Operation_Normal:
	sub 80h
	jp z,Parse_Add
	dec a
	jr z,Parse_Subtract
	dec a
	jr z,Parse_Multiply
	dec a
	jr z,Parse_Divide
	dec a
	jr z,Parse_Exponent
	cp 8
	jr z,Parse_Negate
	cp kSquare-84h
	jr z,Parse_Square
	cp kSqrt-84h
	jr z,Parse_SquareRoot
	cp kALog-84h
	jr z,Parse_10X
	cp kLog-84h
	jr z,Parse_Log
Parse_Square:
	call Parse_OneNumber_Start
	ld d,h
	ld e,l
	call BCD_Multiply
	jp Parse_OneValue_End
Parse_SquareRoot:
	call Parse_OneNumber_Start
	call Square_Root
	jp Parse_OneValue_End
Parse_10X:
	call Parse_OneNumber_Start
	call TenX
	jp Parse_OneValue_End
Parse_Log:
	call Parse_TwoNumber_Start
	call Log
	jp Parse_TwoValue_End
Parse_Negate:
	ld a,(rpnEntryCount)
	or a
	jp z,StackError
	ld hl,(rpnNextEntry)
	dec hl
	ld d,(hl)
	dec hl
	ld e,(hl)
	ex de,hl
	inc hl
	ld a,10000000b
	xor (hl)
	ld (hl),a
	jp Interface_Next
Parse_Subtract:
	call Parse_TwoNumber_Start
	call BCD_Subtract
	jp Parse_TwoValue_End
Parse_Multiply:
	call Parse_TwoNumber_Start
	call BCD_Multiply
	jp Parse_TwoValue_End
Parse_Divide:
	call Parse_TwoNumber_Start
	call BCD_Division
	jp Parse_TwoValue_End
Parse_Exponent:
	call Parse_TwoNumber_Start
	call Exponent
	jp Parse_TwoValue_End
Parse_Add:
	;Take the two numbers in the stack...add them...put the result in the top of the stack
	call Parse_TwoNumber_Start
	call BCD_Add
	jp Parse_TwoValue_End
Parse_Square_Data:
	DW 1
	DB 02h
StackError:
	ld a,eArguments
	ld sp,(errorStack)
	ld hl,(errorPointer)
	jp (hl)
ParseError:
	ld a,eParse
	ld sp,(errorStack)
	ld hl,(errorPointer)
	jp (hl)
Parse_TwoNumber_Start:
	ld a,(rpnEntryCount)
	cp 2
	jr c,StackError
	B_CALL RunIndicOn
	set indicOnly,(iy+indicFlags)
	ld hl,(rpnNextEntry)
	dec hl
	ld d,(hl)
	dec hl
	ld e,(hl)
	dec hl
	ld a,(hl)
	dec hl
	ld l,(hl)
	ld h,a
	ret
Parse_OneNumber_Start:
	ld a,(rpnEntryCount)
	cp 1
	jr c,StackError
	B_CALL RunIndicOn
	set indicOnly,(iy+indicFlags)
	ld hl,(rpnNextEntry)
	dec hl
	ld d,(hl)
	dec hl
	ld e,(hl)
	ex de,hl
	ret
Parse_OneValue_End:
	;for routines with only one value on the stack
	push hl
	ld hl,(rpnNextEntry)
Parse_Value_Nowinprogress:
	;fix memory leak
	dec hl
	dec hl
	;
	dec hl
	ld d,(hl)
	dec hl
	ld e,(hl)
	;DE is the starting point of the stack value not to overwrite
	;Check if we read below the stack...if so there weren't enough values, and set DE
	;to the top of memory, namely mathAreaEnd
	ld bc,rpnStackStart
	or a
	sbc hl,bc
	push af
	add hl,bc
	pop af
	jr nc,Parse_OneValue_SkipStackLook
	ld de,(mathAreaEnd)
	inc de
Parse_OneValue_SkipStackLook:
	pop hl
	ld c,(hl)
	inc hl
	ld b,(hl)
	res 7,b
	dec hl
	inc bc
	inc bc
	push hl
	ex de,hl
	or a
	sbc hl,bc			;HL would be the starting point...one more check
	;ld de,(areaFreeStart)
	;or a
	;sbc hl,de
	;jp c,MemoryError				;memory error
	;add hl,de
	ex de,hl
	pop hl
	push de
	;
	add hl,bc
	dec hl
	ex de,hl
	add hl,bc
	dec hl
	ex de,hl
	lddr			;copy it
	pop de
	ld hl,(rpnNextEntry)
	dec hl
	ld (hl),d
	dec hl
	ld (hl),e
	ld b,d
	ld c,e
	dec bc
	ld (areaFreeEnd),bc
	;DE is the newly created number
	jp Interface_Next
Parse_TwoValue_End:
	;for routines with only one value on the stack
	push hl
	ld hl,rpnEntryCount
	dec (hl)
	ld hl,(rpnNextEntry)
	dec hl
	dec hl
	ld (rpnNextEntry),hl
	jr Parse_Value_Nowinprogress
